#ifndef POKEPLATINUM_UNK_02013B10_H
#define POKEPLATINUM_UNK_02013B10_H

#include "struct_decls/struct_02013B10_decl.h"
#include "struct_decls/struct_02013BE0_decl.h"

UnkStruct_02013B10 * sub_02013B10(int param0, void * param1, void * param2);
void sub_02013B40(UnkStruct_02013B10 * param0);
void * sub_02013B54(const UnkStruct_02013B10 * param0);
const void * sub_02013B68(const UnkStruct_02013B10 * param0);
void sub_02013B80(UnkStruct_02013B10 * param0, u8 param1);
void sub_02013B94(UnkStruct_02013B10 * param0);
void sub_02013BA8(void);
void sub_02013BB8(const void * param0, void * param1, int param2, int param3);
UnkStruct_02013BE0 * sub_02013BE0(int param0);
void sub_02013C10(UnkStruct_02013BE0 * param0, u8 param1, u8 param2, u16 param3, fx32 param4, s16 param5, u32 param6, u32 param7, u32 param8);
void sub_02013CA4(UnkStruct_02013BE0 * param0, u8 param1, u8 param2, u16 param3, fx32 param4, s16 param5, u32 param6, u32 param7, u32 param8);
void sub_02013D38(UnkStruct_02013BE0 * param0);
void sub_02013D74(UnkStruct_02013BE0 * param0);
void * sub_02013D94(UnkStruct_02013BE0 * param0);
void sub_02013DA4(UnkStruct_02013BE0 * param0);
void sub_02013DDC(UnkStruct_02013BE0 * param0);
void sub_02013DFC(s16 * param0, u32 param1, u16 param2, fx32 param3);

#endif // POKEPLATINUM_UNK_02013B10_H
